#!/usr/local/bin/python

import regex

class Expr:
  def __init__(self, expr):
    self.expr = expr

  def evaluate(self, evalEnv):
    try:
      copyEnv = evalEnv.copy()
      result = eval(self.expr, copyEnv, copyEnv)
      if type(result) == type(''): return result
      else: return `result`
    except:
      return '<eval('+expr+') failed>'

  def __repr__(self):
    return 'Expr:' + self.expr

class Wpre:
  def __init__(self):
    """Construct a Wpre instance. 
    """
    self.patternVar = regex.compile("`\([A-Za-z_][A-Za-z0-9_]*\)")
    self.patternExpr = regex.compile("`(\(.*\))")
    self.patternElse = regex.compile("`\([`()]\)")
    self.rep = []

  def compile(self, line):
    """Compile the string represented by line into an internal form for which one
    or more substitutions may be performed be a call to evaluate.
    """

    self.rep = []
    self.varList = []
    i=0
    while i < len(line):
      if line[i] == '`':
        if self.patternVar.match(line, i) >= 0:
          text = self.patternVar.group(1)
          var = Expr(text)
          self.rep.append(var)
          self.varList.append(text)
          i = i + len(text) + 1
        elif self.patternExpr.match(line, i) >= 0:
          text = self.patternExpr.group(1)
          expr = Expr(text)
          self.rep.append(expr)
          i = i + len(text) + 3
        elif self.patternElse.match(line, i) >= 0:
          text = self.patternElse.group(1)
          i = i + len(text) + 1
      else:
        self.rep.append(line[i])
        i = i + 1

  def evaluate(self, evalEnv):
    """Evaluate the compiled representation of the input line against a dictionary
    represented by evalEnv. This dictionary should have keys representing variables
    that were defined in the input line.
    """
    _result = ''
    for _element in self.rep:
      try:
        _result = _result + _element
      except:
        try:
          _result = _result + _element.evaluate(evalEnv)
        except:
          print 'expression evaluation failed ', _element
        pass
    return _result
       
  def getRep(self):
    """Return the internal (compiled) representation of an input line. This should
    be used by 'experts' only.
    """
    return self.rep

  def getVars(self):
    """Return a list of variables that were defined in the input line.
    """
    return self.varList

  def hasVar(self, var):
    """Return a true if a variable appeared in the input line and false, otherwise.
    """
    return var in self.varList





